@@ -85,7 +85,7 @@ mod not_docs {
   impl deno_ffi::FfiPermissions for Permissions {
     fn check(
       &mut self,
-      _path: &Path,
+      _path: Option<&Path>,
     ) -> Result<(), deno_core::error::AnyError> {
       unreachable!("snapshotting!")
     }
@@ -121,15 +121,7 @@ mod not_docs {
       deno_url::init(),
       deno_tls::init(),
       deno_web::init(deno_web::BlobStore::default(), Default::default()),
-      deno_fetch::init::<Permissions, deno_fetch::DefaultFileFetchHandler>(
-        "".to_owned(),
-        None,
-        None,
-        None,
-        None,
-        None,
-        deno_fetch::DefaultFileFetchHandler, // No enable_file_fetch
-      ),
+      deno_fetch::init::<Permissions>(Default::default()),
       deno_websocket::init::<Permissions>("".to_owned(), None, None),
       deno_webstorage::init(None),
       deno_crypto::init(None),
