@@ -12,6 +12,7 @@ use deno_core::error::JsError;
 use deno_core::futures::channel::mpsc;
 use deno_core::futures::future::poll_fn;
 use deno_core::futures::stream::StreamExt;
+use deno_core::futures::task::AtomicWaker;
 use deno_core::located_script_name;
 use deno_core::serde::Deserialize;
 use deno_core::serde::Serialize;
@@ -37,6 +38,7 @@ use std::cell::RefCell;
 use std::fmt;
 use std::rc::Rc;
 use std::sync::atomic::AtomicBool;
+use std::sync::atomic::AtomicI32;
 use std::sync::atomic::Ordering;
 use std::sync::Arc;
 use std::task::Context;
@@ -113,7 +115,9 @@ pub struct WebWorkerInternalHandle {
   sender: mpsc::Sender<WorkerControlEvent>,
   pub port: Rc<MessagePort>,
   pub cancel: Rc<CancelHandle>,
-  terminated: Arc<AtomicBool>,
+  termination_signal: Arc<AtomicBool>,
+  has_terminated: Arc<AtomicBool>,
+  terminate_waker: Arc<AtomicWaker>,
   isolate_handle: v8::IsolateHandle,
   pub worker_type: WebWorkerType,
 }
@@ -127,7 +131,7 @@ impl WebWorkerInternalHandle {
     //
     // Therefore just treat it as if the worker has terminated and return.
     if sender.is_closed() {
-      self.terminated.store(true, Ordering::SeqCst);
+      self.has_terminated.store(true, Ordering::SeqCst);
       return Ok(());
     }
     sender.try_send(event)?;
@@ -136,7 +140,21 @@ impl WebWorkerInternalHandle {
 
   /// Check if this worker is terminated or being terminated
   pub fn is_terminated(&self) -> bool {
-    self.terminated.load(Ordering::SeqCst)
+    self.has_terminated.load(Ordering::SeqCst)
+  }
+
+  /// Check if this worker must terminate (because the termination signal is
+  /// set), and terminates it if so. Returns whether the worker is terminated or
+  /// being terminated, as with [`Self::is_terminated()`].
+  pub fn terminate_if_needed(&mut self) -> bool {
+    let has_terminated = self.is_terminated();
+
+    if !has_terminated && self.termination_signal.load(Ordering::SeqCst) {
+      self.terminate();
+      return true;
+    }
+
+    has_terminated
   }
 
   /// Terminate the worker
@@ -147,7 +165,7 @@ impl WebWorkerInternalHandle {
     // This function can be called multiple times by whomever holds
     // the handle. However only a single "termination" should occur so
     // we need a guard here.
-    let already_terminated = self.terminated.swap(true, Ordering::SeqCst);
+    let already_terminated = self.has_terminated.swap(true, Ordering::SeqCst);
 
     if !already_terminated {
       // Stop javascript execution
@@ -162,7 +180,9 @@ impl WebWorkerInternalHandle {
 pub struct SendableWebWorkerHandle {
   port: MessagePort,
   receiver: mpsc::Receiver<WorkerControlEvent>,
-  terminated: Arc<AtomicBool>,
+  termination_signal: Arc<AtomicBool>,
+  has_terminated: Arc<AtomicBool>,
+  terminate_waker: Arc<AtomicWaker>,
   isolate_handle: v8::IsolateHandle,
 }
 
@@ -171,7 +191,9 @@ impl From<SendableWebWorkerHandle> for WebWorkerHandle {
     WebWorkerHandle {
       receiver: Rc::new(RefCell::new(handle.receiver)),
       port: Rc::new(handle.port),
-      terminated: handle.terminated,
+      termination_signal: handle.termination_signal,
+      has_terminated: handle.has_terminated,
+      terminate_waker: handle.terminate_waker,
       isolate_handle: handle.isolate_handle,
     }
   }
@@ -188,7 +210,9 @@ impl From<SendableWebWorkerHandle> for WebWorkerHandle {
 pub struct WebWorkerHandle {
   pub port: Rc<MessagePort>,
   receiver: Rc<RefCell<mpsc::Receiver<WorkerControlEvent>>>,
-  terminated: Arc<AtomicBool>,
+  termination_signal: Arc<AtomicBool>,
+  has_terminated: Arc<AtomicBool>,
+  terminate_waker: Arc<AtomicWaker>,
   isolate_handle: v8::IsolateHandle,
 }
 
@@ -204,19 +228,37 @@ impl WebWorkerHandle {
   }
 
   /// Terminate the worker
-  /// This function will set terminated to true, terminate the isolate and close the message channel
+  /// This function will set the termination signal, close the message channel,
+  /// and schedule to terminate the isolate after two seconds.
   pub fn terminate(self) {
-    // A WebWorkerHandle can be terminated / dropped after `self.close()` has
-    // been called inside the worker, but only a single "termination" can occur,
-    // so we need a guard here.
-    let already_terminated = self.terminated.swap(true, Ordering::SeqCst);
+    use std::thread::{sleep, spawn};
+    use std::time::Duration;
 
-    if !already_terminated {
-      // Stop javascript execution
-      self.isolate_handle.terminate_execution();
-    }
+    let schedule_termination =
+      !self.termination_signal.swap(true, Ordering::SeqCst);
 
     self.port.disentangle();
+
+    if schedule_termination && !self.has_terminated.load(Ordering::SeqCst) {
+      // Wake up the worker's event loop so it can terminate.
+      self.terminate_waker.wake();
+
+      let has_terminated = self.has_terminated.clone();
+
+      // Schedule to terminate the isolate's execution.
+      spawn(move || {
+        sleep(Duration::from_secs(2));
+
+        // A worker's isolate can only be terminated once, so we need a guard
+        // here.
+        let already_terminated = has_terminated.swap(true, Ordering::SeqCst);
+
+        if !already_terminated {
+          // Stop javascript execution
+          self.isolate_handle.terminate_execution();
+        }
+      });
+    }
   }
 }
 
@@ -226,11 +268,15 @@ fn create_handles(
 ) -> (WebWorkerInternalHandle, SendableWebWorkerHandle) {
   let (parent_port, worker_port) = create_entangled_message_port();
   let (ctrl_tx, ctrl_rx) = mpsc::channel::<WorkerControlEvent>(1);
-  let terminated = Arc::new(AtomicBool::new(false));
+  let termination_signal = Arc::new(AtomicBool::new(false));
+  let has_terminated = Arc::new(AtomicBool::new(false));
+  let terminate_waker = Arc::new(AtomicWaker::new());
   let internal_handle = WebWorkerInternalHandle {
     sender: ctrl_tx,
     port: Rc::new(parent_port),
-    terminated: terminated.clone(),
+    termination_signal: termination_signal.clone(),
+    has_terminated: has_terminated.clone(),
+    terminate_waker: terminate_waker.clone(),
     isolate_handle: isolate_handle.clone(),
     cancel: CancelHandle::new_rc(),
     worker_type,
@@ -238,7 +284,9 @@ fn create_handles(
   let external_handle = SendableWebWorkerHandle {
     receiver: ctrl_rx,
     port: worker_port,
-    terminated,
+    termination_signal,
+    has_terminated,
+    terminate_waker,
     isolate_handle,
   };
   (internal_handle, external_handle)
@@ -276,6 +324,7 @@ pub struct WebWorkerOptions {
   pub broadcast_channel: InMemoryBroadcastChannel,
   pub shared_array_buffer_store: Option<SharedArrayBufferStore>,
   pub compiled_wasm_module_store: Option<CompiledWasmModuleStore>,
+  pub maybe_exit_code: Option<Arc<AtomicI32>>,
 }
 
 impl WebWorker {
@@ -318,15 +367,15 @@ impl WebWorker {
       deno_console::init(),
       deno_url::init(),
       deno_web::init(options.blob_store.clone(), Some(main_module.clone())),
-      deno_fetch::init::<Permissions, deno_fetch::FsFetchHandler>(
-        options.user_agent.clone(),
-        options.root_cert_store.clone(),
-        None,
-        None,
-        options.unsafely_ignore_certificate_errors.clone(),
-        None,
-        deno_fetch::FsFetchHandler,
-      ),
+      deno_fetch::init::<Permissions>(deno_fetch::Options {
+        user_agent: options.user_agent.clone(),
+        root_cert_store: options.root_cert_store.clone(),
+        unsafely_ignore_certificate_errors: options
+          .unsafely_ignore_certificate_errors
+          .clone(),
+        file_fetch_handler: Rc::new(deno_fetch::FsFetchHandler),
+        ..Default::default()
+      }),
       deno_websocket::init::<Permissions>(
         options.user_agent.clone(),
         options.root_cert_store.clone(),
@@ -361,7 +410,9 @@ impl WebWorker {
           unstable,
           options.unsafely_ignore_certificate_errors.clone(),
         ),
-        ops::os::init(),
+        ops::os::init(Some(options.maybe_exit_code.expect(
+          "Worker has access to OS ops but exit code was not passed.",
+        ))),
         ops::permissions::init(),
         ops::process::init(),
         ops::signal::init(),
@@ -391,7 +442,11 @@ impl WebWorker {
     });
 
     if let Some(server) = options.maybe_inspector_server.clone() {
-      server.register_inspector(main_module.to_string(), &mut js_runtime);
+      server.register_inspector(
+        main_module.to_string(),
+        &mut js_runtime,
+        false,
+      );
     }
 
     let (internal_handle, external_handle) = {
@@ -496,14 +551,16 @@ impl WebWorker {
     wait_for_inspector: bool,
   ) -> Poll<Result<(), AnyError>> {
     // If awakened because we are terminating, just return Ok
-    if self.internal_handle.is_terminated() {
+    if self.internal_handle.terminate_if_needed() {
       return Poll::Ready(Ok(()));
     }
 
+    self.internal_handle.terminate_waker.register(cx.waker());
+
     match self.js_runtime.poll_event_loop(cx, wait_for_inspector) {
       Poll::Ready(r) => {
         // If js ended because we are terminating, just return Ok
-        if self.internal_handle.is_terminated() {
+        if self.internal_handle.terminate_if_needed() {
           return Poll::Ready(Ok(()));
         }
 
